"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process = require("child_process");
const fs = require("fs");
const os = require("os");
const path = require("path");
const pify = require("pify");
function describe() {
    return {
        code: 'exec-ifcopenshell',
        name: 'Execute IfcOpenShell to convert IFC files into Collada files',
        priority: 11,
    };
}
exports.describe = describe;
function shouldProcess(conv, context) {
    return context.assetsPaths &&
        context.assetsPaths.length &&
        context.assetsPaths.some(assetPath => assetPath.toLowerCase().endsWith('.ifc'));
}
exports.shouldProcess = shouldProcess;
function process(conv, context, progress) {
    return __awaiter(this, void 0, void 0, function* () {
        //=> Create a temporary folder for the assets
        const tmpDir = path.resolve(`${os.tmpdir()}/chuck-exec-ifcopenshell-${Date.now()}`);
        yield pify(fs.mkdir)(tmpDir);
        context.convertedAssetsDir = tmpDir;
        context.convertedAssetsPaths = [];
        const conversions = context.assetsPaths.map((assetPath, index) => __awaiter(this, void 0, void 0, function* () {
            if (assetPath.toLowerCase().endsWith('.ifc')) {
                yield progress('convert-start', `Converting "${assetPath}" from IFC to Collada`);
                yield convertAndStoreAssets(context, assetPath, index);
            }
        }));
        //=> Await conversions and check if there are errors.
        let remainingConversions = conversions.length;
        const errors = [];
        return new Promise((resolve, reject) => {
            conversions.forEach((conversion) => {
                conversion.then(() => {
                    if (--remainingConversions === 0) {
                        if (errors.length) {
                            reject(new ConversionError('Error(s) while converting', errors));
                        }
                        else {
                            resolve();
                        }
                    }
                }).catch((err) => {
                    --remainingConversions;
                    errors.push(err);
                });
            });
        });
    });
}
exports.process = process;
function cleanup(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const rms = context.convertedAssetsPaths.map(assetPath => pify(fs.unlink)(assetPath));
        yield Promise.all(rms);
        yield pify(fs.rmdir)(context.convertedAssetsDir);
    });
}
exports.cleanup = cleanup;
class ConversionError extends Error {
    constructor(message, errors) {
        super(message);
        this.errors = errors;
        Object.setPrototypeOf(this, ConversionError.prototype);
    }
}
function convertAndStoreAssets(context, assetPath, index) {
    return __awaiter(this, void 0, void 0, function* () {
        const fileName = path.parse(assetPath).name + '.dae';
        const filePath = path.resolve(`${context.convertedAssetsDir}/${fileName}`);
        return new Promise((resolve, reject) => {
            child_process.execFile('IfcConvert', [assetPath, filePath, '-y', '--unicode', 'escape', '--use-element-hierarchy', '--use-element-types'], (error, stdout) => {
                if (error) {
                    context.convertedAssetsPaths.push(`${filePath}.tmp`);
                    reject(stdout);
                    return;
                }
                context.convertedAssetsPaths.push(filePath);
                context.assetsPaths[index] = filePath;
                resolve(filePath);
            });
        });
    });
}
